# wanha - your pesky URL-guardian for IRC
# Copyright (C) 2006-2007 Tommi Saviranta  <wnd@iki.fi>
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA


package Wanha::Cookie;

use strict;
use warnings FATAL => 'all';
use diagnostics;

use Wanha::DB;

use Wanha::Config;


# Cache exactly one cookie
my %cached_info;
my $cached_sid;



=item get_info
Get session info

0: session id
r: session info
=cut
sub get_info($) {
	my ($sid) = @_;
	my ($query, $result);
	my %ret;

	(defined($sid)) || return undef;

	(defined($cached_sid) && $cached_sid eq $sid) && return %cached_info;

	my $q_age = "EXTRACT('epoch' from now() - set)";
	$query = &Wanha::DB::prepare(
			"SELECT age, urls, latest, reset, " .	# 0-3
				"set, ip, visits, chan, " .	# 4-7
				"latest_date, " .		# 8
				"EXTRACT('epoch' from NOW() - set) " .
					"as age_now " .
			"FROM cookies " .
			"WHERE sid = &sa&");
	$query = &Wanha::DB::set($query, "a", $sid);
	$result = &Wanha::DB::query($query);

	(&Wanha::DB::row_count($result) == 0) && return undef;

	my @data = &Wanha::DB::get_row($result);
	$ret{'age'} = $data[0];
	$ret{'urls'} = $data[1];
	$ret{'latest'} = $data[2];
	$ret{'reset'} = &Wanha::DB::bool_to_int($data[3]);

	$ret{'set'} = $data[4];
	$ret{'ip'} = $data[5];
	$ret{'visits'} = $data[6];
	$ret{'chan'} = $data[7];

	$ret{'latest_date'} = $data[8];
	$ret{'age_now'} = $data[9];

	%cached_info = %ret;
	$cached_sid = $sid;

	return %ret;
}


=item set_visits
Set visit count (and the latest IP maybe)

0: session id
1: visit count
2: IP (or undef)
=cut
sub set_visits($$$) {
	my ($sid, $count, $ip) = @_;
	my $update;

	if (defined($ip)) {
		$update = &Wanha::DB::prepare("UPDATE cookies " .
				"SET visits = &da&, ip = &sb& " .
				"WHERE sid = &sc&");
		$update = &Wanha::DB::set($update,
				"a", $count,
				"b", $ip,
				"c", $sid);
	} else {
		$update = &Wanha::DB::prepare("UPDATE cookies " .
				"SET visits = &da& " .
				"WHERE sid = &sb&");
		$update = &Wanha::DB::set($update,
				"a", $count,
				"b", $sid);
	}
	&Wanha::DB::update($update);

	# Cache new values
	&get_info($sid);
	$cached_info{'count'} = $count;
	$cached_info{'ip'} = $ip if (defined($ip));
}


=item save_settings
Save cookie settings.

0: sid
1: age
2: url count
3: reset (0 = false, 1 = true)
=cut
sub save_settings($$$$) {
	my ($sid, $age, $urls, $reset) = @_;

	# Be sure to cache this session
	&get_info($sid);

	if ($cached_info{'age'} eq $age && $cached_info{'urls'} eq $urls
			&& $reset eq $cached_info{'reset'}) {
		return;
	}

	# Update database
	my $update = &Wanha::DB::prepare("UPDATE cookies " .
			"SET age = &da&, urls = &db&, reset = &bc& " .
			"WHERE sid = &sd&");
	$update = &Wanha::DB::set($update,
			"a", $age,
			"b", $urls,
			"c", $reset,
			"d", $sid);
	&Wanha::DB::update($update);

	# Cache new values
	$cached_info{'age'} = $age;
	$cached_info{'urls'} = $urls;
	$cached_info{'reset'} = $reset;
}


=item set_channel
Set channel to cookie

0: session id
1: channel id
=cut
sub set_channel($$) {
	my ($sid, $chan_id) = @_;
	my $update;

	$update = &Wanha::DB::prepare(
			"UPDATE cookies SET chan = &da& WHERE sid = &sb&");
	$update = &Wanha::DB::set($update,
			"a", $chan_id,
			"b", $sid);
	&Wanha::DB::update($update);

	# Cache new values
	&get_info($sid);
	$cached_info{'chan'} = $chan_id;
}


=item set_latest
Set latest seen URL id

0: session id
1: url id
=cut
sub set_latest($$) {
	my ($sid, $url_id) = @_;
	my $update;

	$update = &Wanha::DB::prepare(
			"UPDATE cookies SET latest = &da& WHERE sid = &sb&");
	$update = &Wanha::DB::set($update,
			"a", $url_id,
			"b", $sid);
	&Wanha::DB::update($update);

	# Cache new values
	&get_info($sid);
	$cached_info{'latest'} = $url_id;
}


=item update_set_date
Update set date to "now".

0: session id
=cut
sub update_set_date($) {
	my ($sid) = @_;
	my ($query, $result);

	# Set date isn't very important here so transaction is not needed
	$query = &Wanha::DB::prepare(
			"UPDATE cookies SET set = now() WHERE sid = &sa&");
	$query = &Wanha::DB::set($query, "a", $sid);
	&Wanha::DB::update($query);

	# Get set date from DB. This is a little overkill...
	$query = &Wanha::DB::prepare(
			"SELECT set FROM cookies WHERE sid = &sa&");
	$query = &Wanha::DB::set($query, "a", $sid);
	$result = &Wanha::DB::query($query);
	my ($set) = &Wanha::DB::get_row($result);

	# Cache new values
	&get_info($sid);
	$cached_info{'set'} = $set;
}


=item get_ignore_chans
Get channels ignored by cookie.

0: sid
@: chan id
=cut
sub get_ignore_chans($) {
	my ($sid) = @_;
	my ($query, $result);
	my @ignore;

	$query = &Wanha::DB::prepare(
			"SELECT chan FROM chan_ignore WHERE sid = &sa&");
	$query = &Wanha::DB::set($query, "a", $sid);
	$result = &Wanha::DB::query($query);
	(&Wanha::DB::row_count($result) == 0) && return ();

	while (my ($chan) = &Wanha::DB::get_row($result)) {
		push(@ignore, $chan);
	}

	return @ignore;
}


=item add
Add new cookie.

0: session id
1: ip
2: url id to write to latest
=cut
sub add($$$) {
	my ($sid, $ip, $url_id) = @_;
	my $update;

	$ip = "0.0.0.0" if (! defined($ip));
	$update = &Wanha::DB::prepare("INSERT INTO cookies " .
			"(sid, latest, ip, age) " .
			"VALUES (&sa&, &db&, &sc&, &dd&)");
	$update = &Wanha::DB::set($update,
			"a", $sid,
			"b", $url_id,
			"c", $ip,
			"d", $Wanha::Config::cookie_age);
	&Wanha::DB::update($update);

	&get_info($sid);
}


=item create_new
Create new cookie.

0: ip
1: latest url id
r: session id
=cut
sub create_new($$) {
	my ($ip, $url_id) = @_;

	my $try = 32;
	my $sid;
	my $used;

	for (1..16) { # try 16 sids
		$try--;
		my ($query, $result);
		my $a = rand() * (2 ** 24);
		my $b = rand() * (2 ** 24);
		my $c = rand() * (2 ** 24);
		$sid = sprintf "%06x%06x%06x", $a, $b, $c;

		$query = &Wanha::DB::prepare(
				"SELECT sid FROM cookies WHERE sid = &sa&");
		$query = &Wanha::DB::set($query, "a", $sid);
		$result = &Wanha::DB::query($query);
		if (&Wanha::DB::row_count($result) == 0) {
			&add($sid, $ip, $url_id);
			return $sid;
		}
	}
	return;
}



return 1;
